from __future__ import absolute_import

from .test_debug import (DebugViewTests, ExceptionReporterTests,
    ExceptionReporterTests, PlainTextReportTests, ExceptionReporterFilterTests,
    AjaxResponseExceptionReporterFilter)
from .test_defaults import DefaultsTests
from .test_i18n import JsI18NTests, I18NTests, JsI18NTestsMultiPackage, JavascriptI18nTests
from .test_shortcuts import ShortcutTests
from .test_specials import URLHandling
from .test_static import StaticHelperTest, StaticUtilsTests, StaticTests
