# from django.db import models


# class Author(models.Model):
#     name = models.CharField(max_length=100)

#     class Meta:
#         ordering = ['name']

#     def __unicode__(self):
#         return self.name
