from django.http.cookie import SimpleCookie, parse_cookie
from django.http.request import (HttpRequest, QueryDict, UnreadablePostError,
    build_request_repr)
from django.http.response import (HttpResponse, StreamingHttpResponse,
    CompatibleStreamingHttpResponse, HttpResponsePermanentRedirect,
    HttpResponseRedirect, HttpResponseNotModified, HttpResponseBadRequest,
    HttpResponseForbidden, HttpResponseNotFound, HttpResponseNotAllowed,
    HttpResponseGone, HttpResponseServerError, Http404, BadHeaderError)
from django.http.utils import fix_location_header, conditional_content_removal
