{% extends "admin/base_site.html" %}
{% load i18n %}

{% block coltype %}colSM{% endblock %}
{% block breadcrumbs %}
<div class="breadcrumbs">
<a href="{% url 'admin:index' %}">{% trans 'Home' %}</a>
&rsaquo; <a href="{% url 'django-admindocs-docroot' %}">{% trans 'Documentation' %}</a>
&rsaquo; {% trans 'Views' %}
</div>
{% endblock %}
{% block title %}{% trans 'Views' %}{% endblock %}

{% block content %}

<h1>{% trans 'View documentation' %}</h1>

{% regroup views|dictsort:"site_id" by site as views_by_site %}

<div id="content-related" class="sidebar">
<div class="module">
<h2>{% trans 'Jump to site' %}</h2>
<ul>
    {% for site_views in views_by_site %}
    <li><a href="#site{{ site_views.grouper.id }}">{{ site_views.grouper.name }}</a></li>
    {% endfor %}
</ul>
</div>
</div>

<div id="content-main">

{% for site_views in views_by_site %}
<div class="module">
<h2 id="site{{ site_views.grouper.id }}">{% blocktrans with site_views.grouper.name as name %}Views by URL on {{ name }}{% endblocktrans %}</h2>

{% for view in site_views.list|dictsort:"url" %}
{% ifchanged %}
<h3><a href="{% url 'django-admindocs-views-detail' view=view.full_name %}">{{ view.url }}</a></h3>
<p class="small quiet">{% blocktrans with view.full_name as name %}View function: {{ name }}{% endblocktrans %}</p>
<p>{{ view.title }}</p>
<hr />
{% endifchanged %}
{% endfor %}
</div>
{% endfor %}
</div>
{% endblock %}
